==> local-booter-23.3.0-331.100-sources.jar: com/xebialabs/deployit/booter/local/GenerateDeployableLocalDescriptor.java <==
package com.xebialabs.deployit.booter.local;

import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.xlplatform.synthetic.GeneratedDeployableSpecification;

import static com.xebialabs.deployit.booter.local.DescriptorVerification.PLACEHOLDERS_FIELD;

public class GenerateDeployableLocalDescriptor extends LocalDescriptor {


    GenerateDeployableLocalDescriptor(Type registeredType, SyntheticLocalDescriptor deployedDescriptor) {
        super(registeredType);
        GeneratedDeployableSpecification generatedDeployableSpec = deployedDescriptor.getGeneratedDeployableDescription();
        addSuperClass(generatedDeployableSpec.getSuperType().toType(descriptorRegistry()));
        initDescription(generatedDeployableSpec, deployedDescriptor.getDescription());
        setVirtual(generatedDeployableSpec.isVirtual());
        initProperties(deployedDescriptor, generatedDeployableSpec.isInheritDefaultValues());
        initHierarchy();
        resolveIcon();
    }

    private void initDescription(GeneratedDeployableSpecification generatedDeployableSpec, String deployedDescription) {
        String description = generatedDeployableSpec.getDescription();
        if (description == null || description.equals("Description unavailable")) {
            setDescription(deployedDescription + " (deployable)");
        } else {
            setDescription(description);
        }

    }

    private void initProperties(Descriptor descriptor, boolean withDefaults) {
        for (PropertyDescriptor deployedPropertyDescriptor : descriptor.getPropertyDescriptors()) {
            final PropertyKind kind = deployedPropertyDescriptor.getKind();
            boolean isReferenceField = kind == PropertyKind.CI || kind == PropertyKind.SET_OF_CI || kind == PropertyKind.LIST_OF_CI;
            if (deployedPropertyDescriptor.isDeployedSpecific() ||
                    isReferenceField ||
                    deployedPropertyDescriptor.isHidden() ||
                    PLACEHOLDERS_FIELD.equals(deployedPropertyDescriptor.getName())) {
                continue;
            }

            LocalPropertyDescriptor generatedDeployablePropertyDescriptor = new GeneratedDeployablePropertyDescriptor(this, deployedPropertyDescriptor);
            if (withDefaults) {
                GlobalContextRegistry.register(generatedDeployablePropertyDescriptor, GlobalContextRegistry.lookup(deployedPropertyDescriptor));
            }
            addPropertyDescriptor(generatedDeployablePropertyDescriptor);
        }
    }

}

==> local-booter-23.3.0-414.113-sources.jar: com/xebialabs/deployit/booter/local/GenerateDeployableLocalDescriptor.java <==
package com.xebialabs.deployit.booter.local;

import com.xebialabs.xlplatform.synthetic.GeneratedDeployableSpecification;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.PropertyKind;

import static com.xebialabs.deployit.booter.local.DescriptorVerification.PLACEHOLDERS_FIELD;

public class GenerateDeployableLocalDescriptor extends LocalDescriptor {

    public GenerateDeployableLocalDescriptor(SyntheticLocalDescriptor deployedDescriptor) {
        GeneratedDeployableSpecification generatedDeployableSpec = deployedDescriptor.getGeneratedDeployableDescription();
        setType(generatedDeployableSpec.getType().toType());
        addSuperClass(generatedDeployableSpec.getSuperType().toType());
        initDescription(generatedDeployableSpec, deployedDescriptor.getDescription());
        setVirtual(generatedDeployableSpec.isVirtual());
        initProperties(deployedDescriptor, generatedDeployableSpec.isInheritDefaultValues());
        initHierarchy();
        resolveIcon();
    }

    private void initDescription(GeneratedDeployableSpecification generatedDeployableSpec, String deployedDescription) {
        String description = generatedDeployableSpec.getDescription();
        if (description == null || description.equals("Description unavailable")) {
            setDescription(deployedDescription + " (deployable)");
        } else {
            setDescription(description);
        }

    }

    private void initProperties(Descriptor descriptor, boolean withDefaults) {
        for (PropertyDescriptor deployedPropertyDescriptor : descriptor.getPropertyDescriptors()) {
            final PropertyKind kind = deployedPropertyDescriptor.getKind();
            boolean isReferenceField = kind == PropertyKind.CI || kind == PropertyKind.SET_OF_CI || kind == PropertyKind.LIST_OF_CI;
            if (deployedPropertyDescriptor.isDeployedSpecific() ||
                    isReferenceField ||
                    deployedPropertyDescriptor.isHidden() ||
                    PLACEHOLDERS_FIELD.equals(deployedPropertyDescriptor.getName())) {
                continue;
            }

            LocalPropertyDescriptor generatedDeployablePropertyDescriptor = new GeneratedDeployablePropertyDescriptor(this, deployedPropertyDescriptor);
            if (withDefaults) {
                GlobalContext.register(generatedDeployablePropertyDescriptor, GlobalContext.lookup(deployedPropertyDescriptor));
            }
            addPropertyDescriptor(generatedDeployablePropertyDescriptor);
        }
    }
}
